@@ -39,9 +39,6 @@ DBusMessage *__connman_error_not_supported(DBusMessage *msg);
 
 int __connman_selftest(void);
 
-int __connman_storage_init(void);
-void __connman_storage_cleanup(void);
-
 int __connman_manager_init(DBusConnection *conn, gboolean compat);
 void __connman_manager_cleanup(void);
 
@@ -55,20 +52,26 @@ int __connman_profile_init(DBusConnection *conn);
 void __connman_profile_cleanup(void);
 
 void __connman_profile_list(DBusMessageIter *iter);
+void __connman_profile_list_services(DBusMessageIter *iter);
+const char *__connman_profile_active(void);
 
 #include <connman/log.h>
 
 int __connman_log_init(gboolean detach, gboolean debug);
 void __connman_log_cleanup(void);
 
+void __connman_toggle_debug(void);
+gboolean __connman_debug_enabled(void);
+
 #include <connman/plugin.h>
 
-int __connman_plugin_init(void);
+int __connman_plugin_init(const char *pattern, const char *exclude);
 void __connman_plugin_cleanup(void);
 
 #include <connman/security.h>
 
-int __connman_security_check_privileges(DBusMessage *message);
+int __connman_security_check_privilege(DBusMessage *message,
+				enum connman_security_privilege privilege);
 
 #include <connman/ipv4.h>
 
@@ -82,17 +85,32 @@ void __connman_resolver_cleanup(void);
 
 int __connman_resolver_selftest(void);
 
+#include <connman/storage.h>
+
+int __connman_storage_init(void);
+void __connman_storage_cleanup(void);
+
+int __connman_storage_init_device();
+int __connman_storage_load_device(struct connman_device *device);
+int __connman_storage_save_device(struct connman_device *device);
+int __connman_storage_init_network();
+int __connman_storage_load_network(struct connman_network *network);
+int __connman_storage_save_network(struct connman_network *network);
+
 #include <connman/driver.h>
 
 void __connman_driver_rescan(struct connman_driver *driver);
 
 #include <connman/element.h>
 
-int __connman_element_init(DBusConnection *conn, const char *device);
+int __connman_element_init(DBusConnection *conn, const char *device,
+							const char *nodevice);
 void __connman_element_start(void);
 void __connman_element_stop(void);
 void __connman_element_cleanup(void);
 
+void __connman_element_initialize(struct connman_element *element);
+
 typedef void (* element_cb_t) (struct connman_element *element,
 							gpointer user_data);
 
@@ -105,14 +123,10 @@ void __connman_element_list(struct connman_element *element,
 int __connman_element_count(struct connman_element *element,
 					enum connman_element_type type);
 
+const char *__connman_element_get_device(struct connman_element *element);
+const char *__connman_element_get_network(struct connman_element *element);
+
 const char *__connman_element_type2string(enum connman_element_type type);
-const char *__connman_element_subtype2string(enum connman_element_subtype type);
-
-const char *__connman_element_policy2string(enum connman_element_policy policy);
-enum connman_element_policy __connman_element_string2policy(const char *policy);
-
-int __connman_element_load(struct connman_element *element);
-int __connman_element_store(struct connman_element *element);
 
 static inline void __connman_element_lock(struct connman_element *element)
 {
@@ -122,9 +136,20 @@ static inline void __connman_element_unlock(struct connman_element *element)
 {
 }
 
+int __connman_element_append_ipv4(struct connman_element *element,
+						DBusMessageIter *dict);
+int __connman_element_set_ipv4(struct connman_element *element,
+				const char *name, DBusMessageIter *value);
+
 int __connman_detect_init(void);
 void __connman_detect_cleanup(void);
 
+int __connman_ipv4_init(void);
+void __connman_ipv4_cleanup(void);
+
+int __connman_connection_init(void);
+void __connman_connection_cleanup(void);
+
 #ifdef HAVE_UDEV
 int __connman_udev_init(void);
 void __connman_udev_cleanup(void);
@@ -144,6 +169,21 @@ static inline void __connman_udev_cleanup(void)
 int __connman_device_init(void);
 void __connman_device_cleanup(void);
 
+void __connman_device_increase_connections(struct connman_device *device);
+void __connman_device_decrease_connections(struct connman_device *device);
+
+void __connman_device_set_network(struct connman_device *device,
+					struct connman_network *network);
+
+void __connman_device_disconnect(struct connman_device *device);
+
+connman_bool_t __connman_device_has_driver(struct connman_device *device);
+
+int __connman_device_set_offlinemode(connman_bool_t offlinemode);
+
+int __connman_profile_add_device(struct connman_device *device);
+int __connman_profile_remove_device(struct connman_device *device);
+
 #include <connman/network.h>
 
 int __connman_network_init(void);
@@ -152,6 +192,22 @@ void __connman_network_cleanup(void);
 void __connman_network_set_device(struct connman_network *network,
 					struct connman_device *device);
 
+int __connman_network_disconnect(struct connman_network *network);
+
+connman_bool_t __connman_network_has_driver(struct connman_network *network);
+
+int __connman_profile_add_network(struct connman_network *network);
+int __connman_profile_remove_network(struct connman_network *network);
+
+#include <connman/notifier.h>
+
+int __connman_notifier_init(void);
+void __connman_notifier_cleanup(void);
+
+void __connman_notifier_device_type_increase(enum connman_device_type type);
+void __connman_notifier_device_type_decrease(enum connman_device_type type);
+void __connman_notifier_offline_mode(connman_bool_t enabled);
+
 #include <connman/rtnl.h>
 
 int __connman_rtnl_init(void);
