@@ -40,51 +40,6 @@ static gchar *device_filter = NULL;
 
 static gboolean started = FALSE;
 
-static struct {
-	enum connman_property_id id;
-	int type;
-	const char *name;
-	const void *value;
-} propid_table[] = {
-	{ CONNMAN_PROPERTY_ID_IPV4_METHOD,
-		DBUS_TYPE_STRING, "IPv4.Method", "dhcp" },
-	{ CONNMAN_PROPERTY_ID_IPV4_ADDRESS,
-		DBUS_TYPE_STRING, "IPv4.Address" },
-	{ CONNMAN_PROPERTY_ID_IPV4_NETMASK,
-		DBUS_TYPE_STRING, "IPv4.Netmask" },
-	{ CONNMAN_PROPERTY_ID_IPV4_GATEWAY,
-		DBUS_TYPE_STRING, "IPv4.Gateway" },
-	{ CONNMAN_PROPERTY_ID_IPV4_BROADCAST,
-		DBUS_TYPE_STRING, "IPv4.Broadcast" },
-	{ CONNMAN_PROPERTY_ID_IPV4_NAMESERVER,
-		DBUS_TYPE_STRING, "IPv4.Nameserver" },
-	{ }
-};
-
-static int propid2type(enum connman_property_id id)
-{
-	int i;
-
-	for (i = 0; propid_table[i].name; i++) {
-		if (propid_table[i].id == id)
-			return propid_table[i].type;
-	}
-
-	return DBUS_TYPE_INVALID;
-}
-
-static const char *propid2name(enum connman_property_id id)
-{
-	int i;
-
-	for (i = 0; propid_table[i].name; i++) {
-		if (propid_table[i].id == id)
-			return propid_table[i].name;
-	}
-
-	return NULL;
-}
-
 static const char *type2string(enum connman_element_type type)
 {
 	switch (type) {
@@ -149,116 +104,6 @@ enum connman_ipv4_method __connman_ipv4_string2method(const char *method)
 		return CONNMAN_IPV4_METHOD_UNKNOWN;
 }
 
-#if 0
-static void append_property(DBusMessageIter *dict,
-				struct connman_property *property)
-{
-	if (property->value == NULL)
-		return;
-
-	switch (property->type) {
-	case DBUS_TYPE_ARRAY:
-		connman_dbus_dict_append_array(dict, property->name,
-			property->subtype, &property->value, property->size);
-		break;
-	case DBUS_TYPE_STRING:
-		connman_dbus_dict_append_variant(dict, property->name,
-					property->type, &property->value);
-		break;
-	default:
-		connman_dbus_dict_append_variant(dict, property->name,
-					property->type, property->value);
-		break;
-	}
-}
-
-static void add_common_properties(struct connman_element *element,
-						DBusMessageIter *dict)
-{
-	const char *address = NULL, *netmask = NULL, *gateway = NULL;
-	GSList *list;
-
-	connman_element_get_value(element,
-				CONNMAN_PROPERTY_ID_IPV4_ADDRESS, &address);
-	connman_element_get_value(element,
-				CONNMAN_PROPERTY_ID_IPV4_NETMASK, &netmask);
-	connman_element_get_value(element,
-				CONNMAN_PROPERTY_ID_IPV4_GATEWAY, &gateway);
-
-	if (element->priority > 0)
-		connman_dbus_dict_append_variant(dict, "Priority",
-					DBUS_TYPE_UINT16, &element->priority);
-
-	if (address != NULL)
-		connman_dbus_dict_append_variant(dict, "IPv4.Address",
-						DBUS_TYPE_STRING, &address);
-	if (netmask != NULL)
-		connman_dbus_dict_append_variant(dict, "IPv4.Netmask",
-						DBUS_TYPE_STRING, &netmask);
-	if (gateway != NULL)
-		connman_dbus_dict_append_variant(dict, "IPv4.Gateway",
-						DBUS_TYPE_STRING, &gateway);
-
-	if (element->wifi.security != NULL) {
-		const char *passphrase = "";
-
-		connman_dbus_dict_append_variant(dict, "WiFi.Security",
-				DBUS_TYPE_STRING, &element->wifi.security);
-
-		if (element->wifi.passphrase != NULL)
-			passphrase = element->wifi.passphrase;
-
-		connman_dbus_dict_append_variant(dict, "WiFi.Passphrase",
-				DBUS_TYPE_STRING, &passphrase);
-	}
-
-	__connman_element_lock(element);
-
-	for (list = element->properties; list; list = list->next) {
-		struct connman_property *property = list->data;
-
-		append_property(dict, property);
-	}
-
-	__connman_element_unlock(element);
-}
-
-static void set_common_property(struct connman_element *element,
-				const char *name, DBusMessageIter *value)
-{
-	GSList *list;
-
-	if (g_str_equal(name, "Priority") == TRUE) {
-		dbus_message_iter_get_basic(value, &element->priority);
-		return;
-	}
-
-	__connman_element_lock(element);
-
-	for (list = element->properties; list; list = list->next) {
-		struct connman_property *property = list->data;
-		const char *str;
-
-		if (g_str_equal(property->name, name) == FALSE)
-			continue;
-
-		if (property->flags & CONNMAN_PROPERTY_FLAG_STATIC)
-			continue;
-
-		property->flags &= ~CONNMAN_PROPERTY_FLAG_REFERENCE;
-
-		if (property->type == DBUS_TYPE_STRING) {
-			dbus_message_iter_get_basic(value, &str);
-			g_free(property->value);
-			property->value = g_strdup(str);
-		} else
-			property->value = NULL;
-	}
-
-	__connman_element_unlock(element);
-}
-#endif
-
 static void emit_element_signal(DBusConnection *conn, const char *member,
 					struct connman_element *element)
 {
@@ -570,6 +415,33 @@ void connman_driver_unregister(struct connman_driver *driver)
 				G_TRAVERSE_ALL, -1, remove_driver, driver);
 }
 
+static void unregister_property(gpointer data)
+{
+	struct connman_property *property = data;
+
+	DBG("property %p", property);
+
+	g_free(property->value);
+	g_free(property);
+}
+
+void __connman_element_initialize(struct connman_element *element)
+{
+	DBG("element %p", element);
+
+	element->refcount = 1;
+
+	element->name    = NULL;
+	element->type    = CONNMAN_ELEMENT_TYPE_UNKNOWN;
+	element->index   = -1;
+	element->enabled = FALSE;
+
+	element->configuring = FALSE;
+
+	element->properties = g_hash_table_new_full(g_str_hash, g_str_equal,
+						g_free, unregister_property);
+}
+
 /**
  * connman_element_create:
  * @name: element name
@@ -589,12 +461,7 @@ struct connman_element *connman_element_create(const char *name)
 
 	DBG("element %p", element);
 
-	element->refcount = 1;
-
-	element->name    = g_strdup(name);
-	element->type    = CONNMAN_ELEMENT_TYPE_UNKNOWN;
-	element->index   = -1;
-	element->enabled = FALSE;
+	__connman_element_initialize(element);
 
 	return element;
 }
@@ -611,24 +478,11 @@ struct connman_element *connman_element_ref(struct connman_element *element)
 
 static void free_properties(struct connman_element *element)
 {
-	GSList *list;
-
 	DBG("element %p name %s", element, element->name);
 
 	__connman_element_lock(element);
 
-	for (list = element->properties; list; list = list->next) {
-		struct connman_property *property = list->data;
-
-		if (!(property->flags & CONNMAN_PROPERTY_FLAG_REFERENCE))
-			g_free(property->value);
-
-		g_free(property->name);
-		g_free(property);
-	}
-
-	g_slist_free(element->properties);
-
+	g_hash_table_destroy(element->properties);
 	element->properties = NULL;
 
 	__connman_element_unlock(element);
@@ -656,7 +510,7 @@ void connman_element_unref(struct connman_element *element)
 	}
 }
 
-int connman_element_add_static_property(struct connman_element *element,
+static int set_static_property(struct connman_element *element,
 				const char *name, int type, const void *value)
 {
 	struct connman_property *property;
@@ -670,10 +524,8 @@ int connman_element_add_static_property(struct connman_element *element,
 	if (property == NULL)
 		return -ENOMEM;
 
-	property->flags = CONNMAN_PROPERTY_FLAG_STATIC;
-	property->id    = CONNMAN_PROPERTY_ID_INVALID;
-	property->name  = g_strdup(name);
-	property->type  = type;
+	property->id   = CONNMAN_PROPERTY_ID_INVALID;
+	property->type = type;
 
 	DBG("name %s type %d value %p", name, type, value);
 
@@ -689,54 +541,15 @@ int connman_element_add_static_property(struct connman_element *element,
 	}
 
 	__connman_element_lock(element);
-	element->properties = g_slist_append(element->properties, property);
-	__connman_element_unlock(element);
 
-	return 0;
-}
-
-int connman_element_set_static_property(struct connman_element *element,
-				const char *name, int type, const void *value)
-{
-	GSList *list;
-
-	DBG("element %p name %s", element, element->name);
-
-	if (type != DBUS_TYPE_STRING && type != DBUS_TYPE_BYTE)
-		return -EINVAL;
-
-	__connman_element_lock(element);
-
-	for (list = element->properties; list; list = list->next) {
-		struct connman_property *property = list->data;
-
-		if (g_str_equal(property->name, name) == FALSE)
-			continue;
-
-		if (!(property->flags & CONNMAN_PROPERTY_FLAG_STATIC))
-			continue;
-
-		property->type = type;
-		g_free(property->value);
-
-		switch (type) {
-		case DBUS_TYPE_STRING:
-			property->value = g_strdup(*((const char **) value));
-			break;
-		case DBUS_TYPE_BYTE:
-			property->value = g_try_malloc(1);
-			if (property->value != NULL)
-				memcpy(property->value, value, 1);
-			break;
-		}
-	}
+	g_hash_table_replace(element->properties, g_strdup(name), property);
 
 	__connman_element_unlock(element);
 
 	return 0;
 }
 
-int connman_element_add_static_array_property(struct connman_element *element,
+static int set_static_array_property(struct connman_element *element,
 			const char *name, int type, const void *value, int len)
 {
 	struct connman_property *property;
@@ -750,9 +563,7 @@ int connman_element_add_static_array_property(struct connman_element *element,
 	if (property == NULL)
 		return -ENOMEM;
 
-	property->flags   = CONNMAN_PROPERTY_FLAG_STATIC;
 	property->id      = CONNMAN_PROPERTY_ID_INVALID;
-	property->name    = g_strdup(name);
 	property->type    = DBUS_TYPE_ARRAY;
 	property->subtype = type;
 
@@ -770,169 +581,16 @@ int connman_element_add_static_array_property(struct connman_element *element,
 	}
 
 	__connman_element_lock(element);
-	element->properties = g_slist_append(element->properties, property);
+
+	g_hash_table_replace(element->properties, g_strdup(name), property);
+
 	__connman_element_unlock(element);
 
 	return 0;
 }
 
-static void *get_reference_value(struct connman_element *element,
-						enum connman_property_id id)
-{
-	GSList *list;
-
-	DBG("element %p name %s", element, element->name);
-
-	for (list = element->properties; list; list = list->next) {
-		struct connman_property *property = list->data;
-
-		if (property->id != id)
-			continue;
-
-		if (!(property->flags & CONNMAN_PROPERTY_FLAG_REFERENCE))
-			return property->value;
-	}
-
-	if (element->parent == NULL)
-		return NULL;
-
-	return get_reference_value(element->parent, id);
-}
-
-static void set_reference_properties(struct connman_element *element)
-{
-	GSList *list;
-
-	DBG("element %p name %s", element, element->name);
-
-	for (list = element->properties; list; list = list->next) {
-		struct connman_property *property = list->data;
-
-		if (!(property->flags & CONNMAN_PROPERTY_FLAG_REFERENCE))
-			continue;
-
-		property->value = get_reference_value(element->parent,
-								property->id);
-	}
-}
-
-static struct connman_property *create_property(struct connman_element *element,
-						enum connman_property_id id)
-{
-	struct connman_property *property;
-	GSList *list;
-
-	DBG("element %p name %s", element, element->name);
-
-	__connman_element_lock(element);
-
-	for (list = element->properties; list; list = list->next) {
-		property = list->data;
-
-		if (property->id == id)
-			goto unlock;
-	}
-
-	property = g_try_new0(struct connman_property, 1);
-	if (property == NULL)
-		goto unlock;
-
-	property->flags = CONNMAN_PROPERTY_FLAG_REFERENCE;
-	property->id    = id;
-	property->name  = g_strdup(propid2name(id));
-	property->type  = propid2type(id);
-
-	if (property->name == NULL) {
-		g_free(property);
-		property = NULL;
-		goto unlock;
-	}
-
-	element->properties = g_slist_append(element->properties, property);
-
-unlock:
-	__connman_element_unlock(element);
-
-	return property;
-}
-
-static void create_default_properties(struct connman_element *element)
-{
-	struct connman_property *property;
-	int i;
-
-	DBG("element %p name %s", element, element->name);
-
-	for (i = 0; propid_table[i].name; i++) {
-		DBG("property %s", propid_table[i].name);
-
-		property = create_property(element, propid_table[i].id);
-
-		property->flags &= ~CONNMAN_PROPERTY_FLAG_REFERENCE;
-
-		if (propid_table[i].type != DBUS_TYPE_STRING)
-			continue;
-
-		if (propid_table[i].value)
-			property->value = g_strdup(propid_table[i].value);
-		else
-			property->value = g_strdup("");
-	}
-}
-
-static int define_properties_valist(struct connman_element *element,
-								va_list args)
-{
-	enum connman_property_id id;
-
-	DBG("element %p name %s", element, element->name);
-
-	id = va_arg(args, enum connman_property_id);
-
-	while (id != CONNMAN_PROPERTY_ID_INVALID) {
-
-		DBG("property %d", id);
-
-		create_property(element, id);
-
-		id = va_arg(args, enum connman_property_id);
-	}
-
-	return 0;
-}
-
-/**
- * connman_element_define_properties:
- * @element: an element
- * @varargs: list of property identifiers
- *
- * Define the valid properties for an element.
- *
- * Returns: %0 on success
- */
-int connman_element_define_properties(struct connman_element *element, ...)
-{
-	va_list args;
-	int err;
-
-	DBG("element %p name %s", element, element->name);
-
-	va_start(args, element);
-
-	err = define_properties_valist(element, args);
-
-	va_end(args);
-
-	return err;
-}
-
-int connman_element_create_property(struct connman_element *element,
-						const char *name, int type)
-{
-	return -EIO;
-}
-
-int connman_element_set_property(struct connman_element *element,
+#if 0
+static int set_property(struct connman_element *element,
 				enum connman_property_id id, const void *value)
 {
 	switch (id) {
@@ -972,6 +630,7 @@ int connman_element_set_property(struct connman_element *element,
 
 	return 0;
 }
+#endif
 
 int connman_element_get_value(struct connman_element *element,
 				enum connman_property_id id, void *value)
@@ -1035,65 +694,25 @@ int connman_element_get_value(struct connman_element *element,
 	return 0;
 }
 
-gboolean connman_element_get_static_property(struct connman_element *element,
+static gboolean get_static_property(struct connman_element *element,
 						const char *name, void *value)
 {
-	GSList *list;
+	struct connman_property *property;
 	gboolean found = FALSE;
 
 	DBG("element %p name %s", element, element->name);
 
 	__connman_element_lock(element);
 
-	for (list = element->properties; list; list = list->next) {
-		struct connman_property *property = list->data;
-
-		if (!(property->flags & CONNMAN_PROPERTY_FLAG_STATIC))
-			continue;
-
-		if (g_str_equal(property->name, name) == TRUE) {
-			switch (property->type) {
-			case DBUS_TYPE_STRING:
-				*((char **) value) = property->value;
-				found = TRUE;
-				break;
-			case DBUS_TYPE_BYTE:
-				memcpy(value, property->value, 1);
-				found = TRUE;
-				break;
-			}
-			break;
-		}
-	}
-
-	__connman_element_unlock(element);
-
-	if (found == FALSE && element->parent != NULL)
-		return connman_element_get_static_property(element->parent,
-								name, value);
-
-	return found;
-}
-
-gboolean connman_element_get_static_array_property(struct connman_element *element,
-					const char *name, void *value, int *len)
-{
-	GSList *list;
-	gboolean found = FALSE;
-
-	DBG("element %p name %s", element, element->name);
-
-	__connman_element_lock(element);
-
-	for (list = element->properties; list; list = list->next) {
-		struct connman_property *property = list->data;
-
-		if (!(property->flags & CONNMAN_PROPERTY_FLAG_STATIC))
-			continue;
-
-		if (g_str_equal(property->name, name) == TRUE) {
+	property = g_hash_table_lookup(element->properties, name);
+	if (property != NULL) {
+		switch (property->type) {
+		case DBUS_TYPE_STRING:
 			*((char **) value) = property->value;
-			*len = property->size;
+			found = TRUE;
+			break;
+		case DBUS_TYPE_BYTE:
+			memcpy(value, property->value, 1);
 			found = TRUE;
 			break;
 		}
@@ -1101,77 +720,256 @@ gboolean connman_element_get_static_array_property(struct connman_element *eleme
 
 	__connman_element_unlock(element);
 
+	if (found == FALSE && element->parent != NULL)
+		return get_static_property(element->parent, name, value);
+
 	return found;
 }
 
-gboolean connman_element_match_static_property(struct connman_element *element,
+static gboolean get_static_array_property(struct connman_element *element,
+			const char *name, void *value, unsigned int *len)
+{
+	struct connman_property *property;
+	gboolean found = FALSE;
+
+	DBG("element %p name %s", element, element->name);
+
+	__connman_element_lock(element);
+
+	property = g_hash_table_lookup(element->properties, name);
+	if (property != NULL) {
+		*((char **) value) = property->value;
+		*len = property->size;
+		found = TRUE;
+	}
+
+	__connman_element_unlock(element);
+
+	return found;
+}
+
+#if 0
+static gboolean match_static_property(struct connman_element *element,
 					const char *name, const void *value)
 {
-	GSList *list;
+	struct connman_property *property;
 	gboolean result = FALSE;
 
 	DBG("element %p name %s", element, element->name);
 
 	__connman_element_lock(element);
 
-	for (list = element->properties; list; list = list->next) {
-		struct connman_property *property = list->data;
-
-		if (!(property->flags & CONNMAN_PROPERTY_FLAG_STATIC))
-			continue;
-
-		if (g_str_equal(property->name, name) == FALSE)
-			continue;
-
+	property = g_hash_table_lookup(element->properties, name);
+	if (property != NULL) {
 		if (property->type == DBUS_TYPE_STRING)
 			result = g_str_equal(property->value,
 						*((const char **) value));
-
-		if (result == TRUE)
-			break;
 	}
 
 	__connman_element_unlock(element);
 
 	return result;
 }
+#endif
 
-static void append_connections(DBusMessageIter *entry)
+/**
+ * connman_element_set_string:
+ * @element: element structure
+ * @key: unique identifier
+ * @value: string value
+ *
+ * Set string value for specific key
+ */
+int connman_element_set_string(struct connman_element *element,
+					const char *key, const char *value)
 {
-	DBusMessageIter value, iter;
-	const char *key = "Connections";
-
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-	__connman_element_list(NULL, CONNMAN_ELEMENT_TYPE_CONNECTION, &iter);
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(entry, &value);
+	return set_static_property(element, key, DBUS_TYPE_STRING, &value);
 }
 
-static void emit_connections_signal(DBusConnection *conn)
+/**
+ * connman_element_get_string:
+ * @element: element structure
+ * @key: unique identifier
+ *
+ * Get string value for specific key
+ */
+const char *connman_element_get_string(struct connman_element *element,
+							const char *key)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry;
+	const char *value;
 
-	DBG("conn %p", conn);
+	if (get_static_property(element, key, &value) == FALSE)
+		return NULL;
 
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
+	return value;
+}
 
-	dbus_message_iter_init_append(signal, &entry);
+/**
+ * connman_element_set_uint8:
+ * @element: element structure
+ * @key: unique identifier
+ * @value: integer value
+ *
+ * Set integer value for specific key
+ */
+int connman_element_set_uint8(struct connman_element *element,
+					const char *key, connman_uint8_t value)
+{
+        return set_static_property(element, key, DBUS_TYPE_BYTE, &value);
+}
 
-	append_connections(&entry);
+/**
+ * connman_element_get_uint8:
+ * @element: element structure
+ * @key: unique identifier
+ *
+ * Get integer value for specific key
+ */
+connman_uint8_t connman_element_get_uint8(struct connman_element *element,
+							const char *key)
+{
+	connman_uint8_t value;
 
-	g_dbus_send_message(conn, signal);
+	if (get_static_property(element, key, &value) == FALSE)
+                return 0;
+
+	return value;
+}
+
+/**
+ * connman_element_set_blob:
+ * @element: element structure
+ * @key: unique identifier
+ * @data: blob data
+ * @size: blob size
+ *
+ * Set binary blob value for specific key
+ */
+int connman_element_set_blob(struct connman_element *element,
+			const char *key, const void *data, unsigned int size)
+{
+	return set_static_array_property(element, key,
+						DBUS_TYPE_BYTE, data, size);
+}
+
+/**
+ * connman_element_get_blob:
+ * @element: element structure
+ * @key: unique identifier
+ * @size: pointer to blob size
+ *
+ * Get binary blob value for specific key
+ */
+const void *connman_element_get_blob(struct connman_element *element,
+					const char *key, unsigned int *size)
+{
+	void *value;
+
+	if (get_static_array_property(element, key, &value, size) == FALSE)
+		return NULL;
+
+	return value;
+}
+
+int __connman_element_append_ipv4(struct connman_element *element,
+						DBusMessageIter *dict)
+{
+	const char *method = NULL;
+	const char *address = NULL, *netmask = NULL, *gateway = NULL;
+
+	connman_element_get_value(element,
+				CONNMAN_PROPERTY_ID_IPV4_METHOD, &method);
+
+	connman_element_get_value(element,
+				CONNMAN_PROPERTY_ID_IPV4_ADDRESS, &address);
+	connman_element_get_value(element,
+				CONNMAN_PROPERTY_ID_IPV4_NETMASK, &netmask);
+	connman_element_get_value(element,
+				CONNMAN_PROPERTY_ID_IPV4_GATEWAY, &gateway);
+
+	if (method != NULL)
+		connman_dbus_dict_append_variant(dict, "IPv4.Method",
+						DBUS_TYPE_STRING, &method);
+
+	if (address != NULL)
+		connman_dbus_dict_append_variant(dict, "IPv4.Address",
+						DBUS_TYPE_STRING, &address);
+
+	if (netmask != NULL)
+		connman_dbus_dict_append_variant(dict, "IPv4.Netmask",
+						DBUS_TYPE_STRING, &netmask);
+
+	if (gateway != NULL)
+		connman_dbus_dict_append_variant(dict, "IPv4.Gateway",
+						DBUS_TYPE_STRING, &gateway);
+
+	return 0;
+}
+
+int __connman_element_set_ipv4(struct connman_element *element,
+				const char *name, DBusMessageIter *value)
+{
+	int type;
+
+	type = dbus_message_iter_get_arg_type(value);
+
+	if (g_str_equal(name, "IPv4.Method") == TRUE) {
+		enum connman_ipv4_method method;
+		const char *str;
+
+		if (type != DBUS_TYPE_STRING)
+			return -EINVAL;
+
+		dbus_message_iter_get_basic(value, &str);
+		method = __connman_ipv4_string2method(str);
+		if (method == CONNMAN_IPV4_METHOD_UNKNOWN)
+			return -EINVAL;
+
+		if (method == element->ipv4.method)
+			return -EALREADY;
+
+		element->ipv4.method = method;
+
+		connman_element_update(element);
+	} else if (g_str_equal(name, "IPv4.Address") == TRUE) {
+		const char *address;
+
+		if (type != DBUS_TYPE_STRING)
+			return -EINVAL;
+
+		dbus_message_iter_get_basic(value, &address);
+
+		g_free(element->ipv4.address);
+		element->ipv4.address = g_strdup(address);
+
+		connman_element_update(element);
+	} else if (g_str_equal(name, "IPv4.Netmask") == TRUE) {
+		const char *netmask;
+
+		if (type != DBUS_TYPE_STRING)
+			return -EINVAL;
+
+		dbus_message_iter_get_basic(value, &netmask);
+
+		g_free(element->ipv4.netmask);
+		element->ipv4.netmask = g_strdup(netmask);
+
+		connman_element_update(element);
+	} else if (g_str_equal(name, "IPv4.Gateway") == TRUE) {
+		const char *gateway;
+
+		if (type != DBUS_TYPE_STRING)
+			return -EINVAL;
+
+		dbus_message_iter_get_basic(value, &gateway);
+
+		g_free(element->ipv4.gateway);
+		element->ipv4.gateway = g_strdup(gateway);
+
+		connman_element_update(element);
+	}
+
+	return 0;
 }
 
 static void append_state(DBusMessageIter *entry, const char *state)
@@ -1250,17 +1048,31 @@ static void register_element(gpointer data, gpointer user_data)
 
 	element->path = g_strdup_printf("%s/%s", basepath, element->name);
 
-	set_reference_properties(element);
-
 	__connman_element_unlock(element);
 
 	DBG("element %p path %s", element, element->path);
 
 	g_node_append_data(node, element);
 
+	if (element->type == CONNMAN_ELEMENT_TYPE_DHCP) {
+		element->parent->configuring = TRUE;
+
+		if (__connman_element_count(NULL,
+					CONNMAN_ELEMENT_TYPE_CONNECTION) == 0)
+			emit_state_change(connection, "connecting");
+	}
+
 	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION) {
-		emit_connections_signal(connection);
-		emit_state_change(connection, "online");
+		struct connman_element *parent = element->parent;
+
+		while (parent) {
+			parent->configuring = FALSE;
+			parent = parent->parent;
+		}
+
+		if (__connman_element_count(NULL,
+					CONNMAN_ELEMENT_TYPE_CONNECTION) == 1)
+			emit_state_change(connection, "online");
 	}
 
 	emit_element_signal(connection, "ElementAdded", element);
@@ -1352,7 +1164,6 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 		if (__connman_element_count(NULL,
 					CONNMAN_ELEMENT_TYPE_CONNECTION) == 0)
 			emit_state_change(connection, "offline");
-		emit_connections_signal(connection);
 	}
 
 	emit_element_signal(connection, "ElementRemoved", element);
@@ -1423,10 +1234,13 @@ int connman_element_set_enabled(struct connman_element *element,
 
 	element->enabled = enabled;
 
+	connman_element_update(element);
+
 	return 0;
 }
 
-int __connman_element_init(DBusConnection *conn, const char *device)
+int __connman_element_init(DBusConnection *conn, const char *device,
+							const char *nodevice)
 {
 	struct connman_element *element;
 
@@ -1443,10 +1257,9 @@ int __connman_element_init(DBusConnection *conn, const char *device)
 	element->path = g_strdup("/");
 	element->type = CONNMAN_ELEMENT_TYPE_ROOT;
 
-	create_default_properties(element);
-
 	element_root = g_node_new(element);
 
+	__connman_notifier_init();
 	__connman_network_init();
 	__connman_device_init();
 
@@ -1531,6 +1344,7 @@ void __connman_element_cleanup(void)
 
 	__connman_device_cleanup();
 	__connman_network_cleanup();
+	__connman_notifier_cleanup();
 
 	g_node_traverse(element_root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
 							free_driver, NULL);
