@@ -481,11 +481,9 @@ struct connman_network *connman_network_create(const char *identifier,
 
 	str = type2string(type);
 	if (str != NULL)
-		connman_element_set_static_property(&network->element,
-					"Type", DBUS_TYPE_STRING, &str);
+		connman_element_set_string(&network->element, "Type", str);
 
-	connman_element_set_static_property(&network->element,
-					"Strength", DBUS_TYPE_BYTE, &strength);
+	connman_element_set_uint8(&network->element, "Strength", strength);
 
 	network->type = type;
 	network->identifier = g_strdup(identifier);
@@ -636,47 +634,11 @@ connman_bool_t connman_network_get_available(struct connman_network *network)
 	return network->available;
 }
 
-/**
- * connman_network_set_connected:
- * @network: network structure
- * @connected: connected state
- *
- * Change connected state of network
- */
-int connman_network_set_connected(struct connman_network *network,
-						connman_bool_t connected)
+static gboolean set_connected(gpointer user_data)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "Connected";
+	struct connman_network *network = user_data;
 
-	DBG("network %p connected %d", network, connected);
-
-	if (network->connected == connected)
-		return -EALREADY;
-
-	network->connected = connected;
-
-	if (network->registered == FALSE)
-		return 0;
-
-	signal = dbus_message_new_signal(network->element.path,
-				CONNMAN_NETWORK_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return 0;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN, &connected);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
-
-	if (connected == TRUE) {
+	if (network->connected == TRUE) {
 		struct connman_element *element;
 		enum connman_element_type type = CONNMAN_ELEMENT_TYPE_UNKNOWN;
 
@@ -714,6 +676,53 @@ int connman_network_set_connected(struct connman_network *network,
 		__connman_device_decrease_connections(network->device);
 	}
 
+	return FALSE;
+}
+
+/**
+ * connman_network_set_connected:
+ * @network: network structure
+ * @connected: connected state
+ *
+ * Change connected state of network
+ */
+int connman_network_set_connected(struct connman_network *network,
+						connman_bool_t connected)
+{
+	DBusMessage *signal;
+	DBusMessageIter entry, value;
+	const char *key = "Connected";
+
+	DBG("network %p connected %d", network, connected);
+
+	if (network->connected == connected)
+		return -EALREADY;
+
+	network->connected = connected;
+
+	if (network->registered == FALSE) {
+		g_idle_add(set_connected, network);
+		return 0;
+	}
+
+	signal = dbus_message_new_signal(network->element.path,
+				CONNMAN_NETWORK_INTERFACE, "PropertyChanged");
+	if (signal == NULL)
+		return 0;
+
+	dbus_message_iter_init_append(signal, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
+	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN, &connected);
+	dbus_message_iter_close_container(&entry, &value);
+
+	g_dbus_send_message(connection, signal);
+
+	set_connected(network);
+
 	return 0;
 }
 
@@ -844,7 +853,7 @@ int connman_network_set_string(struct connman_network *network,
 		network->wifi.passphrase = g_strdup(value);
 	}
 
-	return 0;
+	return connman_element_set_string(&network->element, key, value);
 }
 
 /**
@@ -870,7 +879,7 @@ const char *connman_network_get_string(struct connman_network *network,
 	else if (g_str_equal(key, "WiFi.Passphrase") == TRUE)
 		return network->wifi.passphrase;
 
-	return NULL;
+	return connman_element_get_string(&network->element, key);
 }
 
 /**
@@ -888,13 +897,10 @@ int connman_network_set_uint8(struct connman_network *network,
 
 	if (g_str_equal(key, "Priority") == TRUE)
 		network->priority = value;
-	else if (g_str_equal(key, "Strength") == TRUE) {
+	else if (g_str_equal(key, "Strength") == TRUE)
 		network->strength = value;
-		connman_element_set_static_property(&network->element,
-					"Strength", DBUS_TYPE_BYTE, &value);
-	}
 
-	return 0;
+	return connman_element_set_uint8(&network->element, key, value);
 }
 
 /**
@@ -914,7 +920,7 @@ connman_uint8_t connman_network_get_uint8(struct connman_network *network,
 	else if (g_str_equal(key, "Strength") == TRUE)
 		return network->strength;
 
-	return 0;
+	return connman_element_get_uint8(&network->element, key);
 }
 
 /**
@@ -941,7 +947,7 @@ int connman_network_set_blob(struct connman_network *network,
 			network->wifi.ssid_len = 0;
 	}
 
-	return 0;
+	return connman_element_set_blob(&network->element, key, data, size);
 }
 
 /**
@@ -963,7 +969,7 @@ const void *connman_network_get_blob(struct connman_network *network,
 		return network->wifi.ssid;
 	}
 
-	return NULL;
+	return connman_element_get_blob(&network->element, key, size);
 }
 
 void __connman_network_set_device(struct connman_network *network,
@@ -1051,6 +1057,8 @@ static int network_probe(struct connman_element *element)
 		return err;
 	}
 
+	__connman_profile_add_network(network);
+
 	return 0;
 }
 
@@ -1066,6 +1074,8 @@ static void network_remove(struct connman_element *element)
 	if (network->driver == NULL)
 		return;
 
+	__connman_profile_remove_network(network);
+
 	unregister_interface(element);
 
 	if (network->driver->remove)
